@@ -31,10 +31,6 @@ class ModuleProxy(object):
     def __init__(self, module_name):
         self._module_name = module_name
 
-    def __hasattr__(self, key):
-        mod = __import__(self._module_name, fromlist=[''])
-        return hasattr(mod, key)
-
     def __getattr__(self, key):
         mod = __import__(self._module_name, fromlist=[''])
         return getattr(mod, key)
