@@ -334,7 +334,8 @@ class Nifti1Extension(object):
         size = len(self._mangle(self._content))
         size += 8
         # extensions size has to be a multiple of 16 bytes
-        size += 16 - (size % 16)
+        if size % 16 != 0:
+            size += 16 - (size % 16)
         return size
 
     def __repr__(self):
@@ -911,7 +912,7 @@ class Nifti1Header(SpmAnalyzeHeader):
             Qform code. Only returned if `coded` is True.
         """
         hdr = self._structarr
-        code = hdr['qform_code']
+        code = int(hdr['qform_code'])
         if code == 0 and coded:
             return None, 0
         quat = self.get_qform_quaternion()
@@ -1054,7 +1055,7 @@ class Nifti1Header(SpmAnalyzeHeader):
             Sform code. Only returned if `coded` is True.
         """
         hdr = self._structarr
-        code = hdr['sform_code']
+        code = int(hdr['sform_code'])
         if code == 0 and coded:
             return None, 0
         out = np.eye(4)
