@@ -19,6 +19,7 @@ from .tractogram_file import HeaderWarning, DataWarning
 from .tractogram_file import HeaderError, DataError
 from .tractogram import TractogramItem, Tractogram, LazyTractogram
 from .header import Field
+from .utils import peek_next
 
 MEGABYTE = 1024 * 1024
 
@@ -191,8 +192,16 @@ class TckFile(TractogramFile):
             # Write temporary header that we will update at the end
             self._write_header(f, header)
 
+            # Make sure streamlines are in rasmm.
+            tractogram = self.tractogram.to_world(lazy=True)
+            # Assume looping over the streamlines can be done only once.
+            tractogram = iter(tractogram)
+
             try:
-                first_item = next(iter(self.tractogram))
+                # Use the first element to check
+                #  1) the tractogram is not empty;
+                #  2) quantity of information saved along each streamline.
+                first_item, tractogram = peek_next(tractogram)
             except StopIteration:
                 # Empty tractogram
                 header[Field.NB_STREAMLINES] = 0
@@ -216,9 +225,6 @@ class TckFile(TractogramFile):
                        " alongside points. Dropping: {}".format(keys))
                 warnings.warn(msg, DataWarning)
 
-            # Make sure streamlines are in rasmm.
-            tractogram = self.tractogram.to_world(lazy=True)
-
             for t in tractogram:
                 data = np.r_[t.streamline, self.FIBER_DELIMITER]
                 f.write(data.astype(dtype).tostring())
